@@ -1,87 +1,90 @@
-# coding=UTF-8
+# -*- coding: utf-8 -*-
+__author__ = 'vahid'
 
 
 from datetime import timedelta, time, datetime
 from time import struct_time
-from algorithms import  get_julian_day_from_gregorian, \
-                        jalali_date_from_julian_days, \
-                        gregorian_date_from_julian_day, \
-                        parse
-                        
-from jalali_date import JalaliDate, MINYEAR, MAXYEAR, _replace_if_match
+from .algorithms import get_julian_day_from_gregorian, \
+    jalali_date_from_julian_day, \
+    gregorian_date_from_julian_day, \
+    parse
+
+from .jalali_date import JalaliDate, MINYEAR, MAXYEAR
+from khayyam.helpers import replace_if_match
+
+AM_PM = {0: u'ق.ظ',
+         1: u'ب.ظ'}
 
-AM_PM = {0:u'ق.ظ',
-       1:u'ب.ظ'}
 
 class JalaliDatetime(JalaliDate):
-    
     def __init__(self, year=1, month=1, day=1, hour=0, minute=0, second=0, microsecond=0, tzinfo=None):
         if isinstance(year, JalaliDate):
             jd = year
             year = jd.year
             month = jd.month
             day = jd.day
-            
+
         JalaliDate.__init__(self, year, month, day)
         self._time = time(hour, minute, second, microsecond, tzinfo)
-        
-    ##################
+
+    # #################
     ### Properties ###
     ##################
-    
-    
+
+
     def get_hour(self):
         return self._time.hour
+
     def set_hour(self, val):
         self._time.hour = val
-    hour = property(get_hour, set_hour)
 
+    hour = property(get_hour, set_hour)
 
     def get_minute(self):
         return self._time.minute
+
     def set_minute(self, val):
         self._time.minute = val
+
     minute = property(get_minute, set_minute)
 
     def get_second(self):
         return self._time.second
+
     def set_second(self, val):
         self._time.second = val
+
     second = property(get_second, set_second)
 
     def get_microsecond(self):
         return self._time.microsecond
+
     def set_microsecond(self, val):
         self._time.microsecond = val
+
     microsecond = property(get_microsecond, set_microsecond)
-    
+
     def get_tzinfo(self):
         return self._time.tzinfo
+
     def set_tzinfo(self, val):
         self._time.tzinfo = val
+
     tzinfo = property(get_tzinfo, set_tzinfo)
-            
-    
+
+
     #####################
     ### Class Methods ###
     #####################
-    
+
     @classmethod
     def from_datetime(cls, dt, tz=None):
         julian_days = get_julian_day_from_gregorian(dt.year, dt.month, dt.day)
-        arr = jalali_date_from_julian_days(julian_days)
+        arr = jalali_date_from_julian_day(julian_days)
         if not tz:
             tz = dt.tzinfo
         return cls(arr[0], arr[1], arr[2], dt.hour, dt.minute, dt.second, dt.microsecond, tz)
 
-    @classmethod
-    def today(cls):
-        """
-        Return the current local datetime, with tzinfo None. This is equivalent to datetime.fromtimestamp(_time._time()). See also now(), fromtimestamp().
-        """
-        return cls.from_datetime(datetime.today())
-
-
     @classmethod
     def now(cls, tz=None):
         """
@@ -90,19 +93,19 @@ class JalaliDatetime(JalaliDate):
         Else tz must be an instance of a class tzinfo subclass, and the current date and _time are converted to tz's _time zone. In this case the result is equivalent to tz.fromutc(datetime.utcnow().replace(tzinfo=tz)). See also today(), utcnow().        
         """
         return cls.from_datetime(datetime.now(tz))
-    
+
     @classmethod
     def utcnow(cls):
         """
         Return the current UTC date and _time, with tzinfo None. This is like now(), but returns the current UTC date and _time, as a naive datetime object. See also now().
         """
         return cls.from_datetime(datetime.utcnow())
-        
+
     @classmethod
     def dstnow(cls, tz):
         now = cls.now(tz=tz)
         return now + now.dst()
-        
+
     @classmethod
     def fromtimestamp(cls, timestamp, tz=None):
         """
@@ -113,21 +116,21 @@ class JalaliDatetime(JalaliDate):
         fromtimestamp() may raise ValueError, if the timestamp is out of the range of values supported by the platform C localtime() or gmtime() functions. It's common for this to be restricted to years in 1970 through 2038. Note that on non-POSIX systems that include leap seconds in their notion of a timestamp, leap seconds are ignored by fromtimestamp(), and then it's possible to have two timestamps differing by a second that yield identical datetime objects. See also utcfromtimestamp().
         """
         return cls.from_datetime(datetime.fromtimestamp(timestamp, tz=tz))
-    
+
     @classmethod
     def utcfromtimestamp(cls, timestamp):
         """
         Return the UTC datetime corresponding to the POSIX timestamp, with tzinfo None. This may raise ValueError, if the timestamp is out of the range of values supported by the platform C gmtime() function. It's common for this to be restricted to years in 1970 through 2038. See also fromtimestamp().
         """
         return cls.from_datetime(datetime.utcfromtimestamp(timestamp))
-    
-    @classmethod    
+
+    @classmethod
     def fromordinal(cls, ordinal):
         """
         Return the jalali datetime corresponding to the proleptic Gregorian ordinal, where January 1 of year 1 has ordinal 1. ValueError is raised unless 1 <= ordinal <= datetime.max.toordinal(). The hour, minute, second and microsecond of the result are all 0, and tzinfo is None.
         """
         return cls.from_datetime(datetime.fromordinal(ordinal))
-    
+
     @classmethod
     def combine(cls, date, _time):
         """
@@ -136,8 +139,8 @@ class JalaliDatetime(JalaliDate):
         if isinstance(date, (JalaliDatetime, JalaliDate)):
             date = date.to_datetime()
         return cls.from_datetime(datetime.combine(date, _time))
-        
-    
+
+
     @classmethod
     def strptime(cls, date_string, frmt):
         """
@@ -146,37 +149,39 @@ class JalaliDatetime(JalaliDate):
         '%Y/%m/%d'
         """
         # TODO: Implement full features of python, see: http://docs.python.org/library/datetime.html
-        valid_codes = {'%Y':(4, 'year'),
-                       '%m':(2, 'month'),
-                       '%d':(2, 'day'),
-                       '%H':(2, 'hour'),
-                       '%M':(2, 'minute'),
-                       '%S':(2, 'second'),
-                       '%f':(6, 'microsecond')
-                       }
-        
+        valid_codes = {'%Y': (4, 'year'),
+                       '%m': (2, 'month'),
+                       '%d': (2, 'day'),
+                       '%H': (2, 'hour'),
+                       '%M': (2, 'minute'),
+                       '%S': (2, 'second'),
+                       '%f': (6, 'microsecond')
+        }
+
         return parse(cls, date_string, frmt, valid_codes)
-        
-    
+
+
     ########################
     ### Instance Methods ###
     ########################
-    
-    
+
+
     def to_datetime(self):
-        arr = gregorian_date_from_julian_day(self.to_julianday())
-        return datetime(int(arr[0]), int(arr[1]), int(arr[2]), self.hour, self.minute, self.second, self.microsecond, self.tzinfo)
+        arr = gregorian_date_from_julian_day(self.tojulianday())
+        return datetime(int(arr[0]), int(arr[1]), int(arr[2]), self.hour, self.minute, self.second, self.microsecond,
+                        self.tzinfo)
 
     def date(self):
         return JalaliDate(self.year, self.month, self.day)
-    
+
     def time(self):
         return time(self.hour, self.minute, self.second, self.microsecond)
-    
+
     def timetz(self):
         return time(self.hour, self.minute, self.second, self.microsecond, self.tzinfo)
-    
-    def replace(self, year=None, month=None, day=None, hour=None, minute=None, second=None, microsecond=None, tzinfo=None):
+
+    def replace(self, year=None, month=None, day=None, hour=None, minute=None, second=None, microsecond=None,
+                tzinfo=None):
         result = JalaliDatetime(self.year, self.month, self.day, self.hour, self.minute, self.second, self.microsecond)
         if year: result.year = year
         if month: result.month = month
@@ -187,19 +192,19 @@ class JalaliDatetime(JalaliDate):
         if microsecond: result.microsecond
         if tzinfo: result.tzinfo
         return result
-    
+
     def astimezone(self, tz):
         if self.tzinfo is tz:
             return self
-        utc = (self -self.utcoffset()).replace(tzinfo=tz)
+        utc = (self - self.utcoffset()).replace(tzinfo=tz)
         return tz.fromutc(utc)
-    
+
     def utcoffset(self):
         if self.tzinfo:
             return self.tzinfo.utcoffset(self)
         else:
             return None
-    
+
     def dst(self):
         if self.tzinfo:
             return self.tzinfo.dst(self)
@@ -211,49 +216,47 @@ class JalaliDatetime(JalaliDate):
             return self.tzinfo.tzname()
         else:
             return None
-    
-    def timetuple(self):
-        isdst = -1
-        if self.tzinfo:
-            if self.tzinfo.dst(self):
-                isdst = 1
-            else:
-                isdst = 2
-        return struct_time([
-                        self.year,
-                        self.month,
-                        self.day,
-                        self.hour,
-                        self.minute,
-                        self.second,
-                        self.weekday(),
-                        self.dayofyear(),
-                        isdst])
-        
-    
-    def utctimetuple(self):
-        if not self.tzinfo:
-            return self.timetuple()
-        utc = self -self.utcoffset()
-        return struct_time([
-                        utc.year,
-                        utc.month,
-                        utc.day,
-                        utc.hour,
-                        utc.minute,
-                        utc.second,
-                        utc.weekday(),
-                        utc.dayofyear(),
-                        0])
-        
-    
-    def toordinal(self):
-        raise NotImplementedError('please use jalalidate.to_datetime.toordinal() instead')
-    
+
+    # Removed. By vahid
+    # def timetuple(self):
+    #     isdst = -1
+    #     if self.tzinfo:
+    #         if self.tzinfo.dst(self):
+    #             isdst = 1
+    #         else:
+    #             isdst = 2
+    #     return struct_time([
+    #         self.year,
+    #         self.month,
+    #         self.day,
+    #         self.hour,
+    #         self.minute,
+    #         self.second,
+    #         self.weekday(),
+    #         self.dayofyear(),
+    #         isdst])
+    #
+    #
+    # def utctimetuple(self):
+    #     if not self.tzinfo:
+    #         return self.timetuple()
+    #     utc = self - self.utcoffset()
+    #     return struct_time([
+    #         utc.year,
+    #         utc.month,
+    #         utc.day,
+    #         utc.hour,
+    #         utc.minute,
+    #         utc.second,
+    #         utc.weekday(),
+    #         utc.dayofyear(),
+    #         0])
+
+
     def isoformat(self, sep='T'):
         return self.strftime('%Y-%m-%d' + sep + '%H:%M:%S.%f')
-    
-    def strftime(self, frmt):
+
+    def strftime(self, format):
         """
 =========    =======
 Directive    Meaning
@@ -274,7 +277,7 @@ Directive    Meaning
 %p            Locale’s equivalent of either AM or PM.    (2)
 %S            Second as a decimal number [00,61].    (3)
 %U            Week number of the year (Sunday as the first day of the week) as a decimal number [00,53]. All days in a new year preceding the first Sunday are considered to be in week 0.    (4)
-%w            Weekday as a decimal number [0(Sunday),6].     
+%w            Weekday as a decimal number [0(Saturday),6(Friday)].
 %W            Week number of the year (Monday as the first day of the week) as a decimal number [00,53]. All days in a new year preceding the first Monday are considered to be in week 0.    (4)
 %x            Locale’s appropriate date representation.     
 %X            Locale’s appropriate time representation.     
@@ -284,31 +287,31 @@ Directive    Meaning
 %Z            Time zone name (empty string if the object is naive).     
 %%            A literal '%' character.
 =========    =======
-"""     
-   
-        result = super(JalaliDatetime, self).strftime(frmt) 
-        
-        result = _replace_if_match(result, '%H', self.hour)
-        
-        result = _replace_if_match(result, '%I', self.hour12)
-        
-        result = _replace_if_match(result, '%M', self.minute)
-        
-        result = _replace_if_match(result, '%S', self.second)
-        
-        result = _replace_if_match(result, '%f', self.microsecond)
-        
-        result = _replace_if_match(result, '%c', self.localshortformat)
-        result = _replace_if_match(result, '%C', self.localformat)
-        
-        result = _replace_if_match(result, '%p', self.ampm)
-        
-        result = _replace_if_match(result, '%X', self.localtimeformat)
-        
-        result = _replace_if_match(result, '%z', self.utcoffsetformat)
-        
-        result = _replace_if_match(result, '%Z', self.tznameformat)
-        
+"""
+
+        result = super(JalaliDatetime, self).strftime(format)
+
+        result = replace_if_match(result, '%H', self.hour)
+
+        result = replace_if_match(result, '%I', self.hour12)
+
+        result = replace_if_match(result, '%M', self.minute)
+
+        result = replace_if_match(result, '%S', self.second)
+
+        result = replace_if_match(result, '%f', self.microsecond)
+
+        result = replace_if_match(result, '%c', self.localshortformat)
+        result = replace_if_match(result, '%C', self.localformat)
+
+        result = replace_if_match(result, '%p', self.ampm)
+
+        result = replace_if_match(result, '%X', self.localtimeformat)
+
+        result = replace_if_match(result, '%z', self.utcoffsetformat)
+
+        result = replace_if_match(result, '%Z', self.tznameformat)
+
         return result
 
     def localshortformat(self):
@@ -316,20 +319,20 @@ Directive    Meaning
 
     def localformat(self):
         return self.strftime('%A %d %B %Y %I:%M:%S %p')
-    
+
     def localtimeformat(self):
         return self.strftime('%I:%M:%S %p')
-    
+
     def hour12(self):
-        if self.hour > 12 :
+        if self.hour > 12:
             return self.hour - 12
-        return self.hour 
-    
+        return self.hour
+
     def ampm(self):
-        if self.hour < 12 :
+        if self.hour < 12:
             return AM_PM[0]
         return AM_PM[1]
-    
+
     def utcoffsetformat(self):
         if self.tzinfo:
             td = self.utcoffset()
@@ -348,20 +351,20 @@ Directive    Meaning
 
     def __repr__(self):
         return 'khayyam.JalaliDatetime(%s, %s, %s, %s, %s, %s, %s%s)' % \
-            (self.year, self.month, self.day, self.hour, self.minute, self.second, self.microsecond, \
-             ', tzinfo=%s' % self.tzinfo if self.tzinfo else '')
+               (self.year, self.month, self.day, self.hour, self.minute, self.second, self.microsecond, \
+                ', tzinfo=%s' % self.tzinfo if self.tzinfo else '')
 
 
     #################
     ### Operators ###
     #################
-        
+
     def __add__(self, x):
         if isinstance(x, timedelta):
             return JalaliDatetime.from_datetime(self.to_datetime() + x)
-        
+
         raise ValueError('JalaliDatetime object can added by timedelta or JalaliDate object')
-        
+
     def __sub__(self, x):
         if isinstance(x, timedelta):
             return JalaliDatetime.from_datetime(self.to_datetime() - x)
@@ -370,13 +373,12 @@ Directive    Meaning
         elif isinstance(x, JalaliDate):
             return self.to_datetime() - JalaliDatetime(x).to_datetime()
 
-        
         raise ValueError('JalaliDatetime object can added by timedelta, JalaliDatetime or JalaliDate object')
-    
+
     def __lt__(self, x):
         assert isinstance(x, JalaliDatetime), 'Comparison just allow with JalaliDate'
         return self.to_datetime() < x.to_datetime()
-        
+
     def __le__(self, x):
         assert isinstance(x, JalaliDatetime), 'Comparison just allow with JalaliDatetime'
         return self.to_datetime() <= x.to_datetime()
@@ -386,22 +388,22 @@ Directive    Meaning
             return False
         assert isinstance(x, JalaliDatetime), 'Comparison just allow with JalaliDatetime'
         return self.to_datetime() == x.to_datetime()
-    
+
     def __ne__(self, x):
         if not x:
             return True
         assert isinstance(x, JalaliDatetime), 'Comparison just allow with JalaliDatetime'
-        return self.to_datetime() <> x.to_datetime()
-        
+        return self.to_datetime() != x.to_datetime()
+
     def __gt__(self, x):
         assert isinstance(x, JalaliDatetime), 'Comparison just allow with JalaliDatetime'
         return self.to_datetime() > x.to_datetime()
-    
+
     def __ge__(self, x):
         assert isinstance(x, JalaliDatetime), 'Comparison just allow with JalaliDate'
         return self.to_datetime() >= x.to_datetime()
-    
-## Class attributes
+
+# # Class attributes
 JalaliDatetime.min = JalaliDatetime(MINYEAR, 1, 1)
 JalaliDatetime.max = JalaliDatetime(MAXYEAR, 12, 29, 23, 59, 59, 999999)
 JalaliDatetime.resolution = timedelta(microseconds=1)
